@@ -4,11 +4,11 @@ use File::Path 'mkpath';
 use File::Copy 'copy';
 use Config;
 
-my $VERSION = '1.09';			# Changelog at end
+my $VERSION = '1.10';			# Changelog at end
 die "Debugging cycle detected"		# set to -1 to allow extra iteration
   if ++$ENV{PERL_DEBUG_MCODE_CYCLE} > 1;
 
-my %opt;
+my(@ARGVi, %opt) = @ARGV;
 $opt{$1} = shift while ($ARGV[0] || 0) =~ /^-([dq1OBU])$/;
 if ($opt{1}) {
   open STDERR, '>&STDOUT' or warn "can't redirect STDERR to STDOUT";
@@ -58,7 +58,8 @@ Assumptions:
 	no dependence on its position in its parent distribution, if any).
 
 Creates a subdirectory ./$bd0 or  ./${bd0}O.  Add them to `clean' in Makefile.PL
-(add also the temporary files triggering running this script, if applicable).
+(add also the temporary files triggering running this script, if applicable), like
+./had-tst-run-<TESTNAME>.
 
 			Version: $VERSION
 EOP
@@ -86,6 +87,11 @@ chdir '..' or die "chdir ..: $!"
 # In principle, it is easier to do in the "trigger" code with proper BEGIN/END;
 # just be extra careful, and recheck. (And we can be used standalone as well!)
 
+warn <<EOW;
+
+     >>>	Trying to auto-debug (v=$VERSION): $0 @ARGVi
+
+EOW
 # There are 4 cases detected below, with !@ARGV thrown in, one covers 8 types.
 my($skip_makefiles, $mod_load_out);
 if ($chk_module) {
@@ -170,7 +176,20 @@ if ($opt{B}) {
 }
 
 my $build_was_OK = -f "$bd/autodebug-make-ok";
-die "Directory $bd exist; won't overwrite" if -d $bd and not ($opt{U} and $build_was_OK);
+if (-d $bd) {
+  if ($build_was_OK) {
+    die "Directory $bd exists; won't overwrite (give option -U to override)" unless $opt{U};
+    warn <<EOW
+Directory $bd exists with a good build; reusing per option -U.
+
+	(If you edited something in the "top" directory "in between",
+	 you need to copy changes to the directory $bd manually!!!)
+
+EOW
+  } else {
+    warn "Directory $bd exists, but the build was not successful; retrying...";
+  }
+}
 mkdir $bd or die "mkdir $bd: $!" unless -d $bd;
 chdir $bd or die "chdir $bd: $!";
 
@@ -223,35 +242,93 @@ die $postpone if $postpone;	# Reached without a debugger only with -B
 
 my $p = ($^X =~ m([\\/]) ? $^X : `which perl`) || $^X;
 chomp $p unless $p eq $^X;
-my(@cmd, $ver, $ver_done, $cand_done, $dscript);
+my(@cmd, $ver, $ver_done, $cand_done, $dscript, $dbbatchname);
+my $leak_ok = 1 eq ($ENV{AUTO_DBG_LOADMAPS} || 0);
 
 for my $script (@ARGV) {
   $script = "../$script" if not -f $script and -f "../$script";
   if ($gdb) {
     $ver = $gdb;
-    my $gdb_in = 'gdb-in';
-    open TT, ">$gdb_in" or die "Can't open $gdb_in for write: $!";
+    my $gdb_in = $dbbatchname = 'gdb-in';
     # bt full: include local vars (not in 5.0; is in 6.5; is in 6.1, but crashes on 6.3:
 	# http://www.cpantesters.org/cpan/report/2fffc390-afd2-11df-834b-ae20f5ac70d3)
     # disas /m : with source lines (FULL function?!) (not in 6.5; is in 7.0.1)
     # XXX all-registers may take 6K on amd64; maybe put at end?
     # sharedlibrary: present on 7.3.1 (2011)
-    my $proc = (-d "/proc/$$" ? <<EOP : '');	# Not working on cygwin 2016, gdb v7.8??? Move to end
+    my $graceful_degradation;
+    my $tst_python = q[gdb -nx -batch -ex "python print(' py' + 'thon-ok ')"];
+    my $out = `$tst_python 2>&1`;
+    if ($out =~ /\bpython-ok\s*/) {
+      $graceful_degradation = <<'EOP';
+python
+import os
+
+def safe(*cmds):
+    for cmd in cmds:
+        try:
+            gdb.execute(cmd)
+            return
+        except gdb.error as e:
+            print(f"[SKIPPED] {cmd}: {e}")
+    print("[FAILED] No fallback command succeeded.")
+
+safe("set confirm off")
+
+safe("bt full 5", "bt 5 full", "bt full", "bt")
+print("\n=====================================\n\n")
+
+safe("disassemble /m")
+print("\n=====================================\n\n")
+
+safe("info proc status")
+print("\n=====================================\n\n")
+
+if os.getenv("AUTO_DBG_LOADMAPS") != "1":
+    print("[INFO] Memory maps disabled to reduce attack surface; enable by AUTO_DBG_LOADMAPS=1.")
+    print("\n=====================================\n\n")
+else:
+    safe("info sharedlibrary", "info shared")
+    print("\n=====================================\n\n")
+
+    safe("info proc mapping")
+    print("\n=====================================\n\n")
+
+    safe("info files")
+    print("\n=====================================\n\n")
+
+
+safe("info w32 thread-information-block")
+print("\n=====================================\n\n")
+
+safe("info w32 exception")
+print("\n=====================================\n\n")
+
+safe("info w32 ldt")
+print("\n=====================================\n\n")
+
+gdb.execute("quit")
+end
+EOP
+    } else {
+      warn <<EOW;
+Seems gdb is not supporting embedded-Python.  Tested by <<<$tst_python 2>&1>>> ==>
+$out
+
+  I'll try using GDB commands directly.
+  Since GDB interpreter isn't backward-compatible, some commands may
+  fail - and then this would ruin the rest of my script!
+
+EOW
+      my $proc = (-d "/proc/$$" ? <<EOP : '');	# Not working on cygwin 2016, gdb v7.8??? Move to end
 info proc mapping
 echo \\n=====================================\\n\\n
 EOP
-    my $extra = '';
-    $extra .= <<EOE if $^O =~ /cygwin|MSWin/;	# Present on 7.3.1 (2011)
+      my $extra = '';
+      $extra .= <<EOE if $^O =~ /cygwin|MSWin/;	# Present on 7.3.1 (2011)
 info w32 thread-information-block
 echo \\n=====================================\\n\\n
 EOE
-    print TT ($dscript = <<EOP);		# Slightly different order than dbx...
-run -Mblib $script
-echo \\n=====================================\\n\\n
-bt
-echo \\n=====================================\\n\\n
-info all-registers
-echo \\n=====================================\\n\\n
+      $graceful_degradation = <<EOP;	# Not yet used
 disassemble
 echo \\n=====================================\\n\\n
 bt 5 full
@@ -262,12 +339,25 @@ ${extra}info sharedlibrary
 echo \\n=====================================\\n\\n
 ${proc}quit
 EOP
-    close TT or die "Can't close $gdb_in for write: $!";
+    }
+    $dscript = <<EOP;		# Slightly different order than dbx...
+run -Mblib $script
+echo \\n=====================================\\n\\n
+bt
+echo \\n=====================================\\n\\n
+info all-registers
+echo \\n=====================================\\n\\n
+$graceful_degradation
+EOP
+
+      # One could also consider
+      # info frame; info threads; thread apply all bt full 5; info target
 
     #open STDIN, $gdb_in or die "cannot open STDIN from $gdb_in: $!";
     @cmd = (qw(gdb -batch), "--command=$gdb_in", $p);
   } elsif ($lldb) {
     $ver = $lldb;
+    my $Python = `lldb -b -o 'script print(" pyth" + "on-ok ")' -o 'quit' 2>&1`;
     warn <<EOW;
 
 !!!!  I seem to have found LLDB, but extra work may be needed.  !!!
@@ -294,23 +384,38 @@ EOP
 !!!!  I'm crossing my virtual fingers and proceed.              !!!
 
 EOW
-    my $lldb_in = 'lldb-in';
-    open TT, ">$lldb_in" or die "Can't open $lldb_in for write: $!";
+    my $lldb_in = $dbbatchname = 'lldb-in';
     # bt full: include local vars (not in 5.0; is in 6.5; is in 6.1, but crashes on 6.3:
 	# http://www.cpantesters.org/cpan/report/2fffc390-afd2-11df-834b-ae20f5ac70d3)
     # disas /m : with source lines (FULL function?!) (not in 6.5; is in 7.0.1)
     # XXX all-registers may take 6K on amd64; maybe put at end?
     # sharedlibrary: present on 7.3.1 (2011)
-    my $proc = (-d "/proc/$$" ? <<EOP : '');	# Not working on cygwin 2016, gdb v7.8??? Move to end
-script print "??? info proc mapping"
+    my $proc = $leak_ok && <<'EOP';
+image list
 script print "\\n=====================================\\n"
+
+image dump sections 
+script print "\n=====================================\n"
+
+## script print "??? info proc mapping"
+script
+import lldb
+process = lldb.debugger.GetSelectedTarget().process
+count = process.GetNumMemoryRegions()
+for i in range(count):
+    region = lldb.SBMemoryRegionInfo()
+    process.GetMemoryRegionAtIndex(i, region)
+    print(region)
+end
+
+script print "\n=====================================\n"
 EOP
     my $extra = '';
     $extra .= <<EOE if $^O =~ /cygwin|MSWin/;	# Present on 7.3.1 (2011)
 script print "??? info w32 thread-information-block"
 script print "\\n=====================================\\n"
 EOE
-    print TT ($dscript = <<EOP);		# Slightly different order than dbx...
+    $dscript = <<EOP;		# Slightly different order than dbx...
 run -Mblib $script
 script print "\\n=====================================\\n"
 bt
@@ -321,19 +426,14 @@ register read
 script print "\\n=====================================\\n"
 disassemble --frame
 script print "\\n=====================================\\n"
-bt 5 full
+bt --count 5 --verbose		# bt 5 full
 script print "\\n=====================================\\n"
 disassemble --frame --mixed
 script print "\\n=====================================\\n"
-image list
-script print "\\n=====================================\\n"
-image dump sections 
-script print "\\n=====================================\\n"
 register read --all
 script print "\\n=====================================\\n"
 ${extra}${proc}quit
 EOP
-    close TT or die "Can't close $lldb_in for write: $!";
 
     #open STDIN, $gdb_in or die "cannot open STDIN from $gdb_in: $!";
     @cmd = (qw(lldb -batch -s), $lldb_in, $p);
@@ -347,15 +447,22 @@ EOP
     # regs [-f] [-F]        # Print value of registers (-f/-F: SPARC only)
     # list -<n>             # List previous <n> lines (next with +)
     #   -i or -instr        # Intermix source lines and assembly code
+    my $pmap = $leak_ok && '/usr/proc/bin/pmap -F \$proc;';
     @cmd = ($dbxname, qw(-c),		# We do not do non-integer registers...
-	    qq(run -Mblib $script; echo; echo =================================; echo; where -v; echo; echo =================================; echo; dump; echo; echo =================================; echo; regs; echo; echo =================================; echo; list -i +1; echo; echo =================================; echo; list -i -10; echo; echo =================================; echo; echo ============== up 1:; up; dump; echo; echo ============== up 2:; up; dump; echo; echo ============== up 3:; up; dump; echo; echo ============== up 4:; up; dump; echo ==============; /usr/proc/bin/pmap -F \$proc; quit),
+	    qq(run -Mblib $script; echo; echo =================================; echo; where -v; echo; echo =================================; echo; dump; echo; echo =================================; echo; regs; echo; echo =================================; echo; list -i +1; echo; echo =================================; echo; list -i -10; echo; echo =================================; echo; echo ============== up 1:; up; dump; echo; echo ============== up 2:; up; dump; echo; echo ============== up 3:; up; dump; echo; echo ============== up 4:; up; dump; echo ==============; $pmap quit),
 	    $p);
   }
+  if (defined $dbbatchname) {
+    open TT, ">$dbbatchname" or die "Can't open $dbbatchname for write: $!";
+    print TT $dscript;
+    close TT or die "Can't close $dbbatchname for write: $!";
+    warn "\n========= $dbbatchname script begin\n\n$dscript\n\n========= script end\n\n";
+  }
   warn "\nDebugger's version: $ver\n" unless $ver_done++;
   warn 'Running {{{', join('}}} {{{', @cmd), "}}}\n\n";
   if (system @cmd) {
     warn "Running @cmd: rc=$?", ($dscript ? "\n========= script begin\n$dscript\n========= script end\n\t" : '');
-    find_candidates();
+    find_candidates() if $leak_ok;
     die "I stop here,"
   }
 }
@@ -402,3 +509,10 @@ __END__
 1.08	Rename Makefile.PL in the subdirectory (to avoid recursive calls from Makefile.PL in the parent directory).
 	Remove the corresponding warning from ./README too.
 1.09	A match against "Free Software" was broken.
+1.10  Start to code graceful degradation of version-tuned commands for GDB (via python).  XXXX Untested!!!
+	Fix spelling in a warning and emit version and @ARGV info.
+	More detailed logging of variants of reuse of an existing dbg-build directory.
+	Add a few more GDB 'info' entries with global info (in the python-branch).
+	(Temporarily???) Make DLL load-memory-mapping in GDB opt-in via the environment variable AUTO_DBG_LOADMAPS=1.
+	Likewise for dbx (untested) and lldb (untested).
+	Likewise for inspection of candidates for a debugger on a failure of debug script.
