@@ -1,6 +1,6 @@
 package Math::PariBuild;
 
-$VERSION = '2.030520';
+$VERSION = '2.030524';
 
 require Exporter;
 @ISA = 'Exporter';
@@ -593,7 +593,8 @@ sub patches_for ($) {
 			      'patches/diff_2.1.7_restart'],
 		 '2.3.*' =>  [
 			($^O =~ /^MSWin32\b/ ? 'patches/diff_2.3.5_mingw-w64' : ()),
-			      'patches/diff_2.3.5_stderr_clobber'], 
+			      'patches/diff_2.3.5_stderr_clobber',
+			      'patches/diff-2.3.5-buffer-overflow'],
 		 '2.5.0' =>  [@p250 = qw(patches/diff-2.5.0-x-prototype
 				         patches/diff-2.5.0-exceptions
 				         patches/diff-2.5.0-tune_opt)],		# Well, tune_opt is symptomatic: the bug is in PARI
@@ -878,8 +879,9 @@ sub write_paricfg {
 
 EOP
   my $shellq = ($^O eq 'os2' or $^O =~ /win32/i or $^O eq 'dos') ? q(") : q(');
-  my $datadir = '/usr/local/lib/pari/';
-  warn "Trying to find DATADIR of an installed version of GP/PARI (if such exists)...\n";
+  my $datadir = '/opt/local/lib/pari/';
+  $datadir = '/usr/local/lib/pari/' unless -d $datadir;
+  warn "Trying to find DATADIR of an installed version of GP/PARI (if such exists)...\n\t\tIt is OK to see 'sh: gp: command not found' on the next line...\n";
   { local %ENV;
     delete $ENV{GP_DATA_DIR};
     (my $o = `$^X -wle "print shift" "print(default(datadir))" | gp -q` || '')
